use strict;
use Math::Trig;

# settings
my $opacity = 0.04; # opacity of points

my $pointsize = 0.2; # size of point sprites

my @rgb = (1, 0.03, 0); # RGB value for point sprites

my @points = (); # this stores the points

# x-values for main curve
my @radii = (2.224859546, 2.373621301, 2.512487625, 2.628939476, 2.68328126, 2.628513541, 2.511906625, 2.372978616, 2.22418602, 2.070078867, 1.91271945, 1.753196903);

# z-values for main curve
my @heights = (0, 0.072067241, 0.161732679, 0.27904727, 0.435158504, 0.591120817, 0.708281277, 0.797851107, 0.869854643, 0.929643702, 0.980257534, 1.023573845);

my $stages = scalar @radii;

# populate the array with points
# first, the main hourglass/cap
for (my $stage = 0; $stage < $stages; $stage++) {
    # calculate number of points for each ring
    # keep in mind that spacing between rings ~= 0.11
    my $radius = @radii[$stage];
    my $height = @heights[$stage];
    my $numpoints = int(2 * pi * $radius / 0.11);

    # some points will need to be larger
    my $extrasize = 0;
    $extrasize = 0.2 if ($stage < 6 or $stage == 16);
    
    # add points for each "ring"
	for (my $i = 0; $i < $numpoints; $i++) {
        my $theta = ($i + 0.5 * ($stage % 2)) / ($numpoints) * 2 * pi;

        # randomly change point sizes so it looks more natural
		my $rand = rand(0.2);
        
        # add random "jitter" to the points
        my $randx = rand(0.03)-0.015;
        my $randy = rand(0.03)-0.015;
        my $randz = rand(0.03)-0.015;
        
		my $x = $radius * cos($theta);
		my $y = $radius * sin($theta);
		push @points, [$x+$randx, $y+$randy, $height+$randz, $pointsize+$extrasize+$rand, $rgb[0], $rgb[1], $rgb[2]];
	}
}

# print out CMOD definition
print "#celmodel__ascii\n";
print "material\n";
print "   opacity $opacity\n";
print "   texture0 \"27px.png\"\n";
print "   blend add\n";
print "end_material\n";
print "\n";
print "mesh\n";
print "vertexdesc\n";
print "position f3\n";
print "pointsize f1\n";
print "color0 f3\n";
print "end_vertexdesc\n";
print "\n";

my $total = 2 * scalar @points; # number of total points
my @spritelist = (0..$total-1);

print "vertices $total\n";

for (my $j = 0; $j < $total / 2; $j++) {
	printf '%.6f %.6f %.6f %.3f %s %s %s', $points[$j][0], $points[$j][1], $points[$j][2], $points[$j][3], $points[$j][4], $points[$j][5], $points[$j][6];
	print "\n";
	
	# print other side
	printf '%.6f %.6f %.6f %.3f %s %s %s', -1*$points[$j][0], -1*$points[$j][1], -1*$points[$j][2], $points[$j][3], $points[$j][4], $points[$j][5], $points[$j][6];
	print "\n";
}

print "\n";
print "sprites 0 $total\n";
# print join(" ", @spritelist);
for (my $j = 0; $j < scalar @spritelist; $j++) {
    print "@spritelist[$j]";
    if ((@spritelist[$j] % 20 == 0 && @spritelist[$j] != 0) or (@spritelist[$j] > 1000 && @spritelist[$j] % 10 == 0)) {
        print "\n";
    } else {
        print " ";
    }
}
print "\n\n";
print "end_mesh\n";
